@@ -297,13 +297,22 @@ static struct php_mimeheader_with_attributes *php_mimeheader_alloc_from_tok(php_
 	return attr;
 }
 
+static void php_mimepart_free_child(zval *childpart_z)
+{
+	php_mimepart *part = (php_mimepart *)zend_fetch_resource(Z_RES_P(childpart_z), php_mailparse_msg_name(), php_mailparse_le_mime_part());
+	if (part != NULL) {
+		part->parent = NULL;
+		zend_list_delete(part->rsrc);
+	}
+}
+
 PHP_MAILPARSE_API php_mimepart *php_mimepart_alloc()
 {
 	php_mimepart *part = ecalloc(1, sizeof(php_mimepart));
 
 	part->part_index = 1;
 
-	zend_hash_init(&part->children, 0, NULL, NULL, 0);
+	zend_hash_init(&part->children, 0, NULL, (dtor_func_t)php_mimepart_free_child, 0);
 
 	array_init(&part->headerhash);
 
@@ -318,15 +327,7 @@ PHP_MAILPARSE_API php_mimepart *php_mimepart_alloc()
 
 PHP_MAILPARSE_API void php_mimepart_free(php_mimepart *part)
 {
-	zval *childpart_z;
-	HashPosition pos;
-
 	/* free contained parts */
-	zend_hash_internal_pointer_reset_ex(&part->children, &pos);
-	while ((childpart_z = zend_hash_get_current_data_ex(&part->children, &pos)) != NULL) {
-		zval_ptr_dtor(childpart_z);
-		zend_hash_move_forward_ex(&part->children, &pos);
-	}
 
 	zend_hash_destroy(&part->children);
 
@@ -611,9 +612,7 @@ static int php_mimepart_process_line(php_mimepart *workpart)
 
 			newpart = alloc_new_child_part(workpart, workpart->endpos + origcount, 1);
 			php_mimepart_update_positions(workpart, workpart->endpos + origcount, workpart->endpos + linelen, 1);
-			if (workpart->mime_version) {
-				newpart->mime_version = estrdup(workpart->mime_version);
-			}
+			newpart->mime_version = estrdup(workpart->mime_version);
 			newpart->parsedata.in_header = 1;
 			return SUCCESS;
 		}
@@ -914,14 +913,11 @@ static int filter_into_work_buffer(int c, void *dat)
 
 PHP_MAILPARSE_API void php_mimepart_decoder_prepare(php_mimepart *part, int do_decode, php_mimepart_extract_func_t decoder, void *ptr)
 {
-	const mbfl_encoding *encoding;
 	enum mbfl_no_encoding from = mbfl_no_encoding_8bit;
 
 	if (do_decode && part->content_transfer_encoding) {
-		encoding = mbfl_name2encoding(part->content_transfer_encoding);
-		if (encoding) {
-			from = encoding->no_encoding;
-		} else {
+		from = mbfl_name2no_encoding(part->content_transfer_encoding);
+		if (from == mbfl_no_encoding_invalid) {
 			if (strcasecmp("binary", part->content_transfer_encoding) != 0) {
 				zend_error(E_WARNING, "%s(): mbstring doesn't know how to decode %s transfer encoding!",
 						get_active_function_name(),
@@ -939,12 +935,21 @@ PHP_MAILPARSE_API void php_mimepart_decoder_prepare(php_mimepart *part, int do_d
 		if (from == mbfl_no_encoding_8bit || from == mbfl_no_encoding_7bit) {
 			part->extract_filter = NULL;
 		} else {
+#if PHP_VERSION_ID >= 70300
 			part->extract_filter = mbfl_convert_filter_new(
 					mbfl_no2encoding(from), mbfl_no2encoding(mbfl_no_encoding_8bit),
 					filter_into_work_buffer,
 					NULL,
 					part
 					);
+#else
+			part->extract_filter = mbfl_convert_filter_new(
+					from, mbfl_no_encoding_8bit,
+					filter_into_work_buffer,
+					NULL,
+					part
+					);
+#endif
 		}
 	}
 
